/*
** Copyright (C) 2006 Thadeu Lima de Souza Cascardo <cascardo@minaslivre.org>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include <gnet.h>
#include <glib.h>
#include "proto_detect.h"
#include "jabber.h"

static void proto_connect (net_hook_t* hook)
{
}

static void proto_close (net_hook_t* hook)
{
}

static void proto_write (net_hook_t* hook)
{
}

static void proto_read (net_hook_t* hook, gchar* buffer, size_t len)
{
  net_hook_t* new_hook;
  GString* str;
  str = (GString*) hook->data;
  g_string_append_len (str, buffer, len);
  if (str->len >= 7)
    {
      if (!strncmp (str->str, "<stream", 7) ||
	  !strncmp (str->str, "<?xml ", 6))
	{
	  /* Connection is a Jabber client */
	  g_debug ("Connection from %s is a Jabber client.",
		   hook->conn->hostname);
	  new_hook = jabber_hook_new (hook->conn);
	  new_hook->read (new_hook, str->str, str->len);
	  proto_detect_destroy (hook);
	}
      else
	{
	  g_debug ("Unrecognized protocol from %s.",
		   hook->conn->hostname);
	  gnet_conn_disconnect (hook->conn);
	  gnet_conn_unref (hook->conn);
	  proto_detect_destroy (hook);
	}
    }
}

net_hook_t* proto_detect_new (GConn* conn)
{
  net_hook_t* hook;
  hook = g_slice_new (net_hook_t);
  hook->conn = conn;
  hook->peer = NULL;
  hook->server = FALSE;
  hook->connect = proto_connect;
  hook->close = proto_close;
  hook->write = proto_write;
  hook->read = proto_read;
  hook->data = g_string_sized_new (128);
  gnet_conn_set_callback (hook->conn, nethook_event, hook);
  return hook;
}

void proto_detect_destroy (net_hook_t* hook)
{
  if (hook->data != NULL)
    {
      g_string_free (hook->data, TRUE);
    }
  g_slice_free (net_hook_t, hook);
}
